from typing import Optional

from mcp.server import Server, NotificationOptions
from mcp.server.models import InitializationOptions

import click
import mcp.types as types
import asyncio
import mcp

from .vikingdb import VikingDBConnector


def serve(
    vikingdb_host: str, 
    vikingdb_region: str, 
    vikingdb_ak: str, 
    vikingdb_sk: str,
    collection_name: str,
    index_name: str
) -> Server:
    """
    Encapsulates the connection to a VikingDB server and some  methods to interact with it.
    :param vikingdb_host: The host to use for the VikingDB server.
    :param vikingdb_region: The region to use for the VikingDB server.
    :param vikingdb_ak: The Access Key to use for the VikingDB server.
    :param vikingdb_sk: The Secret Key to use for the VikingDB server.
    :param collection_name: The name of the collection to use.
    :param index_name: The name of the index to use.
    """
    


    server = Server("mcp-server-vikingdb")

    vikingdb = VikingDBConnector(
            vikingdb_host,
            vikingdb_region,
            vikingdb_ak,
            vikingdb_sk,
            collection_name,
            index_name
        )
    
    @server.list_tools()
    async def handle_list_tools() -> list[types.Tool]:
        """
        List available tools.
        Each tool specifies its arguments using JSON Schema validation.
        """
        return [
            types.Tool(
                name="vikingdb-upsert-information",
                description="upsert information to vikingdb for later use",
                inputSchema={
                    "type": "object",
                    "properties": {
                        "information": {
                            "type": "string",
                        },
                    },
                    "required": ["information"],
                },
            ),
            types.Tool(
                name="vikingdb-search-information",
                description=(
                    "Look up information in VikingDB. Use this tool when you need to: \n"
                    " - Search for information by their query \n"
                    " - Access information for further analysis \n"
                    " - Get some personal information about the user"
                ),
                inputSchema={
                    "type": "object",
                    "properties": {
                        "query": {
                            "type": "string",
                            "description": "The query to search for in the vikingdb",
                        },
                    },
                    "required": ["query"],
                },
            ),
        ]

    @server.call_tool()
    async def handle_call_tool(
        name: str, arguments: dict | None
    ) -> list[types.TextContent | types.ImageContent | types.EmbeddedResource]:
        """
        Handle tool execution requests.
        Tools can modify server state and notify clients of changes.
        """
        if name not in ["vikingdb-upsert-information", "vikingdb-search-information"]:
            raise ValueError(f"Unknown tool: {name}")

        if name == "vikingdb-upsert-information":
            if not arguments or "information" not in arguments:
                raise ValueError("Missing required argument 'information'")
            information = arguments["information"]
            await vikingdb.upsert_information(information)
            return [types.TextContent(type="text", text=f"{information} has been added to vikingdb {collection_name}")]
        
        if name == "vikingdb-search-information":
            if not arguments or "query" not in arguments:
                raise ValueError("Missing required argument 'query'")
            query = arguments["query"]
            results = await vikingdb.search_information(query)
            return [types.TextContent(type="text", text=f"Search results for {query}:\n{results}")]
        
    return server
    
    
@click.command()
@click.option(
    "--vikingdb-host",
    envvar="VIKINGDB_HOST",
    required=True,
    help="VIKINGDB_HOST",
    default="api-vikingdb.volces.com",
)

@click.option(
    "--vikingdb-region",
    envvar="VIKINGDB_REGION",
    required=True,
    help="VIKINGDB_REGION",
)
@click.option(
    "--vikingdb-ak",
    envvar="VIKINGDB_AK",
    required=True,
    help="VIKINGDB_AK",
)
@click.option(
    "--vikingdb-sk",
    envvar="VIKINGDB_SK",
    required=True,
    help="VIKINGDB_SK",
)
@click.option(
    "--collection-name",
    envvar="COLLECTION_NAME",
    required=True,
    help="Collection name",
)
@click.option(
    "--index-name",
    envvar="INDEX_NAME",
    required=True,
    help="Index name",
)

def main(
    vikingdb_host: str, 
    vikingdb_region: str,
    vikingdb_ak: str,
    vikingdb_sk: str,
    collection_name: str,
    index_name: str,
):

    async def _run():
        async with mcp.server.stdio.stdio_server() as (read_stream, write_stream):
            server = serve(
                vikingdb_host, 
                vikingdb_region, 
                vikingdb_ak, 
                vikingdb_sk,
                collection_name,
                index_name,
            )
            await server.run(
                read_stream,
                write_stream,
                InitializationOptions(
                    server_name="mcp-server-vikingdb",
                    server_version="0.1.0",
                    capabilities=server.get_capabilities(
                        notification_options=NotificationOptions(),
                        experimental_capabilities={},
                    ),
                ),
            )

    asyncio.run(_run())




from typing import Optional,List,Dict
from dotenv import load_dotenv
import os
import random
from volcengine.viking_db import *



class VikingDBConnector:
    """
    Encapsulates the connection to a VikingDB server and some  methods to interact with it.
    :param vikingdb_host: The host to use for the VikingDB server.
    :param vikingdb_region: The region to use for the VikingDB server.
    :param vikingdb_ak: The Access Key to use for the VikingDB server.
    :param vikingdb_sk: The Secret Key to use for the VikingDB server.
    :param collection_name: The name of the collection to use.
    :param index_name: The name of the index to use.
    """
    
    def __init__(
            self, 
            vikingdb_host: str, 
            vikingdb_region: str, 
            vikingdb_ak: str, 
            vikingdb_sk: str,
            collection_name: str,
            index_name: str
    ):
        self._vikingdb_host = vikingdb_host
        self._vikingdb_region = vikingdb_region
        self._vikingdb_ak = vikingdb_ak
        self._vikingdb_sk = vikingdb_sk
        self._collection_name = collection_name
        self._index_name = index_name
        
        self._client = VikingDBService(
            host=self._vikingdb_host,
            region=self._vikingdb_region,
            ak=self._vikingdb_ak,
            sk=self._vikingdb_sk,
            connection_timeout=100)
        
        self._collection = self._client.async_get_collection(self._collection_name)
        self._index = self._client.get_index(self._collection_name, self._index_name)
        
        
    async def upsert_information(self, information:str):
        self._collection = await self._client.async_get_collection(self._collection_name)
        vector = await  self._client.async_embedding_v2(EmbModel("bge-m3", params={"return_token_usage": True}), [RawData("text", information)])
        vector = vector["sentence_dense_embedding"][0]
        field1 = {"text_id": random.randint(10,100000),"text_content": information,"text_vector":vector}
        data1 = Data(field1)
        datas = []
        datas.append(data1)
        await self._collection.async_upsert_data(datas)  
        
        
    async def search_information(self, query:str):
        self._collection = await self._client.async_get_collection(self._collection_name)
        self._index = await self._client.async_get_index(self._collection_name, self._index_name)
        search_vector = await self._client.async_embedding_v2(EmbModel("bge-m3", params={"return_token_usage": True}), [RawData("text", query)])
        search_vector = search_vector["sentence_dense_embedding"][0]
        search_res = await self._index.async_search_by_vector(search_vector,limit=1) #纯稠密检索
        
        return search_res[0].fields
